--[[	PhantomPlates Personal NamePlate Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

--------------------------
--[[	Constants	]]
--------------------------
local DefaultYPos=-128;

----------------------------------
--[[	CUF NamePlate Options	]]
----------------------------------
local CUF_PersonalNamePlateOptions={
	displaySelectionHighlight=false;
--	displayAggroHighlight=false;
	displayName=false;
	fadeOutOfRange=false;
	colorNameBySelection=true;
	smoothHealthUpdates=false;
--	displayNameWhenSelected=false;
	showLevel=true;

	healthBarColorOverride=CreateColor(0,1,0);
	defaultBorderColor=CreateColor(0,0,0,1);
};

----------------------------------
--[[	Personal NamePlate	]]
----------------------------------
local PersonalNamePlate_Initialized=false;

local PersonalNamePlate=CreateFrame("Frame");
PersonalNamePlate:SetPoint("CENTER",0,DefaultYPos);
PersonalNamePlate:SetSize(128,32);
PersonalNamePlate:Hide();

PersonalNamePlate:RegisterEvent("PLAYER_LOGIN");

PersonalNamePlate:RegisterEvent("PLAYER_REGEN_ENABLED");
PersonalNamePlate:RegisterEvent("PLAYER_REGEN_DISABLED");
PersonalNamePlate:RegisterUnitEvent("UNIT_MAXHEALTH","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_HEALTH","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_HEALTH_FREQUENT","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_BAR_SHOW","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_BAR_HIDE","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_DISPLAYPOWER","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_MAXPOWER","player");
PersonalNamePlate:RegisterUnitEvent("UNIT_POWER_UPDATE","player");

local function PersonalNamePlate_UpdateVisibility(show)
	if not PersonalNamePlate_Initialized then return; end--	Don't run if we haven't loaded yet (ex. ADDON_LOADED fires before PLAYER_LOGIN)

--	Visibility Logic
	local powertype=UnitPowerType("player");
	show=(
		AddOn.Options.PersonalNamePlate_Enabled--	Enabled?
		and	(	AddOn.OptionsPanel:IsVisible()--	Options Panel visible?
			or	show--	Force shown?
			or	(	show==nil--	Omitted arg
				and	(	InCombatLockdown()
					or	UnitHealth("player")<UnitHealthMax("player")--	Default action
					or	(AddOn.Options.PowerBars_Enabled and UnitPower("player",powertype)<UnitPowerMax("player",powertype))
				)
			)
		) and true or false--	Cast to boolean
	);

--	Apply if changed
	if show~=PersonalNamePlate:IsShown() then
		PersonalNamePlate:SetShown(show);
	end
end

PersonalNamePlate:SetScript("OnEvent",function(self,event,...)
	if event=="PLAYER_LOGIN" then--	PersonalNamePlate Init (allow addons hooking NamePlateDriverFrame:OnNamePlateCreated() to work on this)
		NamePlateDriverFrame:OnNamePlateCreated(self);--	Creates UnitFrame and injects NamePlateBaseMixin
		self.UnitFrame.RaidTargetFrame:Hide();--		RaidTargetIcon is always hidden on player

--		Apply Layout (NamePlateDriverFrame won't overwrite this since it never recieves NAME_PLATE_UNIT_ADDED for "player")
		DefaultCompactNamePlateFrameSetupInternal(PersonalNamePlate.UnitFrame
			,DefaultCompactNamePlateFrameSetUpOptions
			,CUF_PersonalNamePlateOptions
		);
		CompactUnitFrame_UpdateAll(PersonalNamePlate.UnitFrame);

--[[		Setup Conversion Notes
--		These are in NamePlateDriverFrame:ApplyFrameOptions()
		self:OnOptionsUpdated();--	Defunct in classic
		NamePlateDriverFrame:UpdateInsetsForType("friendly",self);--	This isn't a real NamePlate

		NamePlateDriverFrame:OnUnitAuraUpdate("player");
		NamePlateDriverFrame:OnRaidTargetUpdate();
--]]

--		Safe to update visibility now
		PersonalNamePlate_Initialized=true;
	end

--	Force shown if PLAYER_REGEN_DISABLED is firing, otherwise use automated logic (InCombatLockdown() isn't true yet)
	PersonalNamePlate_UpdateVisibility(event=="PLAYER_REGEN_DISABLED" or nil);
end);

PersonalNamePlate:SetScript("OnShow",function(self)
	self:OnAdded("player",NamePlateDriverFrame);
	NamePlateDriverFrame:OnUnitAuraUpdate("player");
end);
PersonalNamePlate:SetScript("OnHide",function(self) return self:OnRemoved(); end);

local function OptionsPanel_OnVisibilityChanged() PersonalNamePlate_UpdateVisibility(); end
AddOn.OptionsPanel:HookScript("OnShow",OptionsPanel_OnVisibilityChanged);
AddOn.OptionsPanel:HookScript("OnHide",OptionsPanel_OnVisibilityChanged);

--------------------------
--[[	Options Setup	]]
--------------------------
AddOn.RegisterOptionsControl("PersonalNamePlate_Enabled",false
	,"OnClick","SetChecked","GetChecked"
	,function(_,_,val)--	Callback
		local control_ypos=AddOn.GetOptionsControl("PersonalNamePlate_YPos");
		control_ypos:SetEnabled(val);
		do	local fontobj=val and GameFontHighlightSmall or GameFontDisableSmall;
			control_ypos.Text:SetFontObject(fontobj);
			control_ypos.Low:SetFontObject(fontobj);
			control_ypos.Mid:SetFontObject(fontobj);
			control_ypos.High:SetFontObject(fontobj);
		end

		PersonalNamePlate_UpdateVisibility();
	end
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint("TOPLEFT",panel,"TOP",0,-168);
		button.text:SetText(AddOn.Localization.OptionsSetting_PersonalNamePlate_Enabled);
		return button;
	end
);

AddOn.RegisterOptionsControl("PersonalNamePlate_YPos",DefaultYPos
	,"OnValueChanged","SetValue","GetValue"
	,function(self,_,val)--	Callback
		self.Text:SetFormattedText(AddOn.Localization.OptionsSetting_PersonalNamePlate_YPosFormat,val);
		PersonalNamePlate:SetPoint("CENTER",0,val);
	end
	,function(panel)--	Initialization
		local slider=CreateFrame("Slider",nil,panel,"OptionsSliderTemplate");
		slider:SetPoint("TOPLEFT",panel,"TOP",28,-208);
		slider:SetWidth(256);

		slider:SetMinMaxValues(-384,384);
		slider:SetValue(DefaultYPos);

		slider:SetValueStep(16);
		slider:SetStepsPerPage(1);
		slider:SetObeyStepOnDrag(true);

		slider.Mid=slider:CreateFontString(nil,"ARTWORK","GameFontHighlightSmall");
		slider.Mid:SetPoint("TOP",slider,"BOTTOM",0,3);

		slider.Text:SetFontObject(GameFontHighlightSmall);
		slider.Text:SetFormattedText(AddOn.Localization.OptionsSetting_PersonalNamePlate_YPosFormat,DefaultYPos);
		slider.Low:SetText(-384);
		slider.Mid:SetText(0);
		slider.High:SetText(384);

		OptionsSlider_YPos=slider;
		return slider;
	end
);

AddOn.HookOption("PowerBars_Enabled",function()
	if AddOn.Options.PersonalNamePlate_Enabled then PersonalNamePlate_UpdateVisibility(); end
end);
