--[[	PhantomPlates Threat Glow Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

--------------------------
--[[	Local Variables	]]
--------------------------
local ThreatStatusColors={
	[0]={0.69,0.69,0.69};
	{1,1,0.47};
	{1,0.6,0};
	{1,0,0};
}

------------------
--[[	Hooks	]]
------------------
local ThreatTextures={};
local ThreatGUIDMap={};

local function UpdateThreat(self,...)
	local status=AddOn.Options.ThreatGlow_Enabled and UnitThreatSituation(...);
	if status and status>0 then
		self:SetVertexColor(unpack(ThreatStatusColors[status]));
		self:Show();
	else self:Hide(); end
end

local function UpdateAllThreat()
	for unitframe,threat in pairs(ThreatTextures) do
		local unit=unitframe.displayedUnit;
		if unit then UpdateThreat(threat,unit); end
	end
end

local EventFrame=CreateFrame("frame");
EventFrame:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE");
EventFrame:RegisterEvent("UNIT_THREAT_LIST_UPDATE");
EventFrame:SetScript("OnEvent",function(self,event,unit)
	local threat=ThreatGUIDMap[UnitGUID(unit)];
	if threat then
		if event=="UNIT_THREAT_LIST_UPDATE" then
			UpdateThreat(threat,"player",unit);
		elseif event=="UNIT_THREAT_SITUATION_UPDATE" then
			UpdateThreat(threat,unit);
		end
	end
end);

--	Nameplate Hooks
AddOn.HookNamePlateCreated(function(_,base)
	local unitframe=base.UnitFrame;
	if not ThreatTextures[unitframe] then
		local threat=unitframe.healthBar.border:CreateTexture(nil,"BACKGROUND");
		threat:SetTexture("Interface\\TargetingFrame\\UI-TargetingFrame-Flash");
		threat:SetTexCoord(0,144/256,270/512,302/512);
		threat:SetPoint("CENTER",0,1);
		threat:SetSize(144,32);
		threat:Hide();

		ThreatTextures[unitframe]=threat;
	end
end);

AddOn.HookNamePlateAdded(function(base,unit)
	local threat=ThreatTextures[base.UnitFrame];
	if threat then
		local guid=UnitGUID(unit);
		ThreatGUIDMap[guid],threat.GUID=threat,guid;

		if NamePlateDriverFrame:GetNamePlateTypeFromUnit(unit)=="enemy" then
			UpdateThreat(threat,"player",unit);
		else
			UpdateThreat(threat,unit);
		end
	end
end);

AddOn.HookNamePlateRemoved(function(base)
	local threat=ThreatTextures[base.UnitFrame];
	local guid=threat and threat.GUID;
	if guid then ThreatGUIDMap[guid],threat.GUID=nil,nil; end
end);

--------------------------
--[[	Options Setup	]]
--------------------------
AddOn.RegisterOptionsControl("ThreatGlow_Enabled",true
	,"OnClick","SetChecked","GetChecked"
	,function(_,_,val)
		for unitframe,threat in pairs(ThreatTextures) do
			local unit=unitframe.displayedUnit;
			if unit then
				if NamePlateDriverFrame:GetNamePlateTypeFromUnit(unit)=="enemy" then
					UpdateThreat(threat,"player",unit);
				else
					UpdateThreat(threat,unit);
				end
			end
		end
	end
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint("TOPLEFT",panel,"TOP",0,-48);
		button.text:SetText(AddOn.Localization.OptionsSetting_ThreatGlow_Enabled);
		return button;
	end
);
